@@ -1,17 +1,14 @@
 
-from .finder.core import FinderFactory, NoMatchesError
+from .finder.core import FinderFactory
 from .parser import DoxygenParserFactory, CacheFactory
 from .renderer.rst.doxygen import DoxygenToRstRendererFactoryCreatorConstructor, \
     RstContentCreator, RenderContext
-from .renderer.rst.doxygen.domain import DomainHandlerFactoryCreator, NullDomainHandler
-from .renderer.rst.doxygen.domain import CppDomainHelper, CDomainHelper
 from .renderer.rst.doxygen.filter import FilterFactory, GlobFactory
 from .renderer.rst.doxygen.target import TargetHandlerFactory
 from .renderer.rst.doxygen.mask import MaskFactory, NullMaskFactory, NoParameterNamesMask
 
 from .finder.doxygen.core import DoxygenItemFinderFactoryCreator
-from .finder.doxygen.matcher import ItemMatcherFactory
-from .directive.base import BaseDirective, DoxygenBaseDirective, WarningHandler, create_warning
+from .directive.base import BaseDirective, create_warning
 from .directive.index import DoxygenIndexDirective, AutoDoxygenIndexDirective
 from .directive.file import DoxygenFileDirective, AutoDoxygenFileDirective
 from .process import AutoDoxygenProcessHandle
@@ -20,8 +17,7 @@ from .project import ProjectInfoFactory, ProjectError
 
 from docutils.parsers.rst.directives import unchanged_required, unchanged, flag
 from docutils.statemachine import ViewList
-from docutils.core import publish_from_doctree
-from sphinx.domains.cpp import DefinitionParser
+from sphinx.domains import cpp, c, python
 from sphinx.writers.text import TextWriter
 from sphinx.builders.text import TextBuilder
 
@@ -29,8 +25,6 @@ import docutils.nodes
 import sphinx.addnodes
 import sphinx.ext.mathbase
 
-from StringIO import StringIO
-
 import os
 import fnmatch
 import re
@@ -39,8 +33,7 @@ import collections
 import subprocess
 
 # Somewhat outrageously, reach in and fix a Sphinx regex
-import sphinx.domains.cpp
-sphinx.domains.cpp._identifier_re = re.compile(r'(~?\b[a-zA-Z_][a-zA-Z0-9_]*)\b')
+cpp._identifier_re = re.compile(r'(~?\b[a-zA-Z_][a-zA-Z0-9_]*)\b')
 
 
 class NoMatchingFunctionError(BreatheError):
@@ -96,7 +89,7 @@ class DoxygenFunctionDirective(BaseDirective):
     final_argument_whitespace = True
 
     def __init__(self, node_factory, text_renderer, *args, **kwargs):
-        super(DoxygenFunctionDirective, self).__init__(*args, **kwargs)
+        BaseDirective.__init__(self, *args, **kwargs)
 
         self.node_factory = node_factory
         self.text_renderer = text_renderer
@@ -128,7 +121,8 @@ class DoxygenFunctionDirective(BaseDirective):
         # Extract arguments from the function name.
         args = self.parse_args(args)
 
-        finder_filter = self.filter_factory.create_function_finder_filter(namespace, function_name)
+        finder_filter = self.filter_factory.create_member_finder_filter(
+            namespace, function_name, 'function')
 
         matches = []
         finder.filter_(finder_filter, matches)
@@ -145,7 +139,7 @@ class DoxygenFunctionDirective(BaseDirective):
             )
 
         try:
-            data_object = self.resolve_function(matches, args, project_info)
+            node_stack = self.resolve_function(matches, args, project_info)
         except NoMatchingFunctionError:
             return warning.warn('doxygenfunction: Cannot find function "{namespace}{function}" '
                                 '{tail}')
@@ -164,7 +158,8 @@ class DoxygenFunctionDirective(BaseDirective):
             # reason (failures in the matcher_stack code) so we consolidate them by shoving them in
             # a set to remove duplicates. Should be fixed!
             for i, entry in enumerate(set(error.signatures)):
-                if i: literal_text += '\n'
+                if i:
+                    literal_text += '\n'
                 # Replace new lines with a new line & enough spacing to reach the appropriate
                 # alignment for our simple plain text list
                 literal_text += '- %s' % entry.replace('\n', '\n  ')
@@ -186,17 +181,16 @@ class DoxygenFunctionDirective(BaseDirective):
             )
         filter_ = self.filter_factory.create_outline_filter(self.options)
 
-        mask_factory = NullMaskFactory()
-        return self.render(data_object, project_info, self.options, filter_, target_handler,
-                           mask_factory)
+        return self.render(node_stack, project_info, self.options, filter_, target_handler,
+                           NullMaskFactory())
 
     def parse_args(self, function_description):
         # Strip off trailing qualifiers
         pattern = re.compile(r'''(?<= \)) \s*
-                                 (?: const)? \s*
-                                 (?: volatile)? \s*
-                                 (?: = \s* 0)? \s* $ ''',
-                                 re.VERBOSE)
+                             (?: const)? \s*
+                             (?: volatile)? \s*
+                             (?: = \s* 0)? \s* $ ''',
+                             re.VERBOSE)
 
         function_description = re.sub(pattern,
                                       '',
@@ -237,7 +231,7 @@ class DoxygenFunctionDirective(BaseDirective):
         if len(matches) == 1:
             return matches[0]
 
-        data_object = None
+        node_stack = None
 
         signatures = []
 
@@ -260,20 +254,20 @@ class DoxygenFunctionDirective(BaseDirective):
             signatures.append(signature)
 
             match = re.match(r"([^(]*)(.*)", signature)
-            function, match_args = match.group(1), match.group(2)
+            match_args = match.group(2)
 
             # Parse the text to find the arguments
             match_args = self.parse_args(match_args)
 
             # Match them against the arg spec
             if args == match_args:
-                data_object = entry
+                node_stack = entry
                 break
 
-        if not data_object:
+        if not node_stack:
             raise UnableToResolveFunctionError(signatures)
 
-        return data_object
+        return node_stack
 
 
 class DoxygenClassLikeDirective(BaseDirective):
@@ -310,16 +304,12 @@ class DoxygenClassLikeDirective(BaseDirective):
             warning = create_warning(None, self.state, self.lineno, kind=self.kind)
             return warning.warn('doxygen{kind}: %s' % e)
 
-        matcher_stack = self.matcher_factory.create_matcher_stack(
-            {
-                "compound": self.matcher_factory.create_name_type_matcher(name, self.kind)
-                },
-            "compound"
-            )
+        finder_filter = self.filter_factory.create_compound_finder_filter(name, self.kind)
 
-        try:
-            data_object = finder.find_one(matcher_stack)
-        except NoMatchesError as e:
+        matches = []
+        finder.filter_(finder_filter, matches)
+
+        if len(matches) == 0:
             warning = create_warning(project_info, self.state, self.lineno, name=name,
                                      kind=self.kind)
             return warning.warn('doxygen{kind}: Cannot find class "{name}" {tail}')
@@ -330,7 +320,7 @@ class DoxygenClassLikeDirective(BaseDirective):
         filter_ = self.filter_factory.create_class_filter(name, self.options)
 
         mask_factory = NullMaskFactory()
-        return self.render(data_object, project_info, self.options, filter_, target_handler,
+        return self.render(matches[0], project_info, self.options, filter_, target_handler,
                            mask_factory)
 
 
@@ -393,13 +383,14 @@ class DoxygenContentBlockDirective(BaseDirective):
         if 'content-only' in self.options:
 
             # Unpack the single entry in the matches list
-            (data_object,) = matches
+            (node_stack,) = matches
 
             filter_ = self.filter_factory.create_content_filter(self.kind, self.options)
 
             # Having found the compound node for the namespace or group in the index we want to grab
             # the contents of it which match the filter
-            contents_finder = self.finder_factory.create_finder_from_root(data_object, project_info)
+            contents_finder = self.finder_factory.create_finder_from_root(node_stack[0],
+                                                                          project_info)
             contents = []
             contents_finder.filter_(filter_, contents)
 
@@ -419,9 +410,9 @@ class DoxygenContentBlockDirective(BaseDirective):
             )
         node_list = []
 
-        for data_object in matches:
+        for node_stack in matches:
             renderer_factory = renderer_factory_creator.create_factory(
-                data_object,
+                node_stack,
                 self.state,
                 self.state.document,
                 filter_,
@@ -429,7 +420,7 @@ class DoxygenContentBlockDirective(BaseDirective):
                 )
 
             mask_factory = NullMaskFactory()
-            context = RenderContext([data_object, self.root_data_object], mask_factory)
+            context = RenderContext(node_stack, mask_factory, self.directive_args)
             object_renderer = renderer_factory.create_renderer(context)
             node_list.extend(object_renderer.render())
 
@@ -463,6 +454,12 @@ class DoxygenBaseItemDirective(BaseDirective):
         }
     has_content = False
 
+    def create_finder_filter(self, namespace, name):
+        """Creates a filter to find the node corresponding to this item."""
+
+        return self.filter_factory.create_member_finder_filter(
+            namespace, name, self.kind)
+
     def run(self):
 
         try:
@@ -482,12 +479,12 @@ class DoxygenBaseItemDirective(BaseDirective):
             warning = create_warning(None, self.state, self.lineno, kind=self.kind)
             return warning.warn('doxygen{kind}: %s' % e)
 
+        finder_filter = self.create_finder_filter(namespace, name)
 
-        matcher_stack = self.create_matcher_stack(namespace, name)
+        matches = []
+        finder.filter_(finder_filter, matches)
 
-        try:
-            data_object = finder.find_one(matcher_stack)
-        except NoMatchesError as e:
+        if len(matches) == 0:
             display_name = "%s::%s" % (namespace, name) if namespace else name
             warning = create_warning(project_info, self.state, self.lineno, kind=self.kind,
                                      display_name=display_name)
@@ -498,88 +495,63 @@ class DoxygenBaseItemDirective(BaseDirective):
             )
         filter_ = self.filter_factory.create_outline_filter(self.options)
 
+        node_stack = matches[0]
         mask_factory = NullMaskFactory()
-        return self.render(data_object, project_info, self.options, filter_, target_handler, mask_factory)
+        return self.render(node_stack, project_info, self.options, filter_, target_handler,
+                           mask_factory)
 
 
 class DoxygenVariableDirective(DoxygenBaseItemDirective):
 
     kind = "variable"
 
-    def create_matcher_stack(self, namespace, name):
-
-        return self.matcher_factory.create_matcher_stack(
-            {
-                "compound": self.matcher_factory.create_name_matcher(namespace),
-                "member": self.matcher_factory.create_name_type_matcher(name, self.kind)
-                },
-            "member"
-            )
+    def render(self, node_stack, project_info, options, filter_, target_handler, mask_factory):
+        # Remove 'extern' keyword as Sphinx doesn't support it.
+        definition = node_stack[0].definition
+        extern = 'extern '
+        if definition.startswith(extern):
+            definition = definition[len(extern):]
+        self.directive_args[1] = [definition]
+        return DoxygenBaseItemDirective.render(self, node_stack, project_info, options, filter_,
+                                               target_handler, mask_factory)
 
 
 class DoxygenDefineDirective(DoxygenBaseItemDirective):
 
     kind = "define"
 
-    def create_matcher_stack(self, namespace, name):
-
-        return self.matcher_factory.create_matcher_stack(
-            {
-                "compound": self.matcher_factory.create_name_matcher(namespace),
-                "member": self.matcher_factory.create_name_type_matcher(name, self.kind)
-                },
-            "member"
-            )
-
 
 class DoxygenEnumDirective(DoxygenBaseItemDirective):
 
     kind = "enum"
 
-    def create_matcher_stack(self, namespace, name):
 
-        return self.matcher_factory.create_matcher_stack(
-            {
-                "compound": self.matcher_factory.create_name_matcher(namespace),
-                "member": self.matcher_factory.create_name_type_matcher(name, self.kind)
-                },
-            "member"
-            )
+class DoxygenEnumValueDirective(DoxygenBaseItemDirective):
+
+    kind = "enumvalue"
+
+    def create_finder_filter(self, namespace, name):
+
+        return self.filter_factory.create_enumvalue_finder_filter(name)
 
 
 class DoxygenTypedefDirective(DoxygenBaseItemDirective):
 
     kind = "typedef"
 
-    def create_matcher_stack(self, namespace, name):
-
-        return self.matcher_factory.create_matcher_stack(
-            {
-                "compound": self.matcher_factory.create_name_matcher(namespace),
-                "member": self.matcher_factory.create_name_type_matcher(name, self.kind)
-                },
-            "member"
-            )
-
 
 class DoxygenUnionDirective(DoxygenBaseItemDirective):
 
     kind = "union"
 
-    def create_matcher_stack(self, namespace, name):
+    def create_finder_filter(self, namespace, name):
 
         # Unions are stored in the xml file with their fully namespaced name
         # We're using C++ namespaces here, it might be best to make this file
         # type dependent
         #
         xml_name = "%s::%s" % (namespace, name) if namespace else name
-
-        return self.matcher_factory.create_matcher_stack(
-            {
-                "compound": self.matcher_factory.create_name_type_matcher(xml_name, self.kind)
-                },
-            "compound"
-            )
+        return self.filter_factory.create_compound_finder_filter(xml_name, 'union')
 
 
 # Setup Administration
@@ -619,6 +591,7 @@ class DoxygenDirectiveFactory(object):
         "doxygenvariable": DoxygenVariableDirective,
         "doxygendefine": DoxygenDefineDirective,
         "doxygenenum": DoxygenEnumDirective,
+        "doxygenenumvalue": DoxygenEnumValueDirective,
         "doxygentypedef": DoxygenTypedefDirective,
         "doxygenunion": DoxygenUnionDirective,
         "doxygennamespace": DoxygenNamespaceDirective,
@@ -628,18 +601,18 @@ class DoxygenDirectiveFactory(object):
         }
 
     def __init__(self, node_factory, text_renderer, root_data_object,
-                 renderer_factory_creator_constructor, finder_factory, matcher_factory,
-                 project_info_factory, filter_factory, target_handler_factory):
+                 renderer_factory_creator_constructor, finder_factory,
+                 project_info_factory, filter_factory, target_handler_factory, parser_factory):
 
         self.node_factory = node_factory
         self.text_renderer = text_renderer
         self.root_data_object = root_data_object
         self.renderer_factory_creator_constructor = renderer_factory_creator_constructor
         self.finder_factory = finder_factory
-        self.matcher_factory = matcher_factory
         self.project_info_factory = project_info_factory
         self.filter_factory = filter_factory
         self.target_handler_factory = target_handler_factory
+        self.parser_factory = parser_factory
 
     # TODO: This methods should be scrapped as they are only called in one place. We should just
     # inline the code at the call site
@@ -656,10 +629,10 @@ class DoxygenDirectiveFactory(object):
             self.root_data_object,
             self.renderer_factory_creator_constructor,
             self.finder_factory,
-            self.matcher_factory,
             self.project_info_factory,
             self.filter_factory,
-            self.target_handler_factory
+            self.target_handler_factory,
+            self.parser_factory
             )
 
     def create_struct_directive_container(self):
@@ -668,6 +641,9 @@ class DoxygenDirectiveFactory(object):
     def create_enum_directive_container(self):
         return self.create_directive_container("doxygenenum")
 
+    def create_enumvalue_directive_container(self):
+        return self.create_directive_container("doxygenenumvalue")
+
     def create_typedef_directive_container(self):
         return self.create_directive_container("doxygentypedef")
 
@@ -705,10 +681,10 @@ class DoxygenDirectiveFactory(object):
             self.root_data_object,
             self.renderer_factory_creator_constructor,
             self.finder_factory,
-            self.matcher_factory,
             self.project_info_factory,
             self.filter_factory,
-            self.target_handler_factory
+            self.target_handler_factory,
+            self.parser_factory
             )
 
     def get_config_values(self, app):
@@ -862,6 +838,54 @@ class FileStateCache(object):
             del self.app.env.breathe_file_state[filename]
 
 
+class DomainDirectiveFactory(object):
+    # A mapping from node kinds to cpp domain classes and directive names.
+    cpp_classes = {
+        'class': (cpp.CPPClassObject, 'class'),
+        'struct': (cpp.CPPClassObject, 'class'),
+        'function': (cpp.CPPFunctionObject, 'function'),
+        'friend': (cpp.CPPFunctionObject, 'function'),
+        'slot': (cpp.CPPFunctionObject, 'function'),
+        'enum': (cpp.CPPTypeObject, 'type'),
+        'typedef': (cpp.CPPTypeObject, 'type'),
+        'union': (cpp.CPPTypeObject, 'type'),
+        'namespace': (cpp.CPPTypeObject, 'type'),
+        # Use CPPClassObject for enum values as the cpp domain doesn't have a directive for
+        # enum values and CPPMemberObject requires a type.
+        'enumvalue': (cpp.CPPClassObject, 'member'),
+        'define': (c.CObject, 'macro')
+    }
+
+    python_classes = {
+        'function': (python.PyModulelevel, 'function'),
+        'variable': (python.PyClassmember, 'attribute')
+    }
+
+    @staticmethod
+    def fix_python_signature(sig):
+        def_ = 'def '
+        if sig.startswith(def_):
+            sig = sig[len(def_):]
+        # Doxygen uses an invalid separator ('::') in Python signatures. Replace them with '.'.
+        return sig.replace('::', '.')
+
+    @staticmethod
+    def create(domain, args):
+        if domain == 'c':
+            return c.CObject(*args)
+        if domain == 'py':
+            cls, name = DomainDirectiveFactory.python_classes.get(
+                args[0], (python.PyClasslike, 'class'))
+            args[1] = [DomainDirectiveFactory.fix_python_signature(n) for n in args[1]]
+        else:
+            cls, name = DomainDirectiveFactory.cpp_classes.get(
+                args[0], (cpp.CPPMemberObject, 'member'))
+        # Replace the directive name because domain directives don't know how to handle
+        # Breathe's "doxygen" directives.
+        args = [name] + args[1:]
+        return cls(*args)
+
+
 # Setup
 # -----
 
@@ -886,18 +910,11 @@ def setup(app):
     math_nodes.displaymath = sphinx.ext.mathbase.displaymath
     node_factory = NodeFactory(docutils.nodes, sphinx.addnodes, math_nodes)
 
-    cpp_domain_helper = CppDomainHelper(DefinitionParser, re.sub)
-    c_domain_helper = CDomainHelper()
-    domain_helpers = {"c": c_domain_helper, "cpp": cpp_domain_helper}
-    domain_handler_factory_creator = DomainHandlerFactoryCreator(node_factory, domain_helpers)
-
     rst_content_creator = RstContentCreator(ViewList, textwrap.dedent)
-    default_domain_handler = NullDomainHandler()
     renderer_factory_creator_constructor = DoxygenToRstRendererFactoryCreatorConstructor(
         node_factory,
         parser_factory,
-        default_domain_handler,
-        domain_handler_factory_creator,
+        DomainDirectiveFactory,
         rst_content_creator
         )
 
@@ -907,7 +924,6 @@ def setup(app):
     build_dir = os.path.dirname(app.doctreedir.rstrip(os.sep))
     project_info_factory = ProjectInfoFactory(app.srcdir, build_dir, app.confdir, fnmatch.fnmatch)
     target_handler_factory = TargetHandlerFactory(node_factory)
-    matcher_factory = ItemMatcherFactory()
 
     root_data_object = RootDataObject()
 
@@ -919,10 +935,10 @@ def setup(app):
         root_data_object,
         renderer_factory_creator_constructor,
         finder_factory,
-        matcher_factory,
         project_info_factory,
         filter_factory,
-        target_handler_factory
+        target_handler_factory,
+        parser_factory
         )
 
     DoxygenFunctionDirective.app = app
@@ -947,6 +963,11 @@ def setup(app):
         directive_factory.create_enum_directive_container(),
         )
 
+    app.add_directive(
+        "doxygenenumvalue",
+        directive_factory.create_enumvalue_directive_container(),
+        )
+
     app.add_directive(
         "doxygentypedef",
         directive_factory.create_typedef_directive_container(),
@@ -999,14 +1020,17 @@ def setup(app):
 
     app.add_config_value("breathe_projects", {}, True)
     app.add_config_value("breathe_default_project", "", True)
-    app.add_config_value("breathe_domain_by_extension", {}, True)
+    # Provide reasonable defaults for domain_by_extension mapping. Can be overridden by users.
+    app.add_config_value("breathe_domain_by_extension", {'py': 'py'}, True)
     app.add_config_value("breathe_domain_by_file_pattern", {}, True)
     app.add_config_value("breathe_projects_source", {}, True)
     app.add_config_value("breathe_build_directory", '', True)
     app.add_config_value("breathe_default_members", (), True)
     app.add_config_value("breathe_implementation_filename_extensions", ['.c', '.cc', '.cpp'], True)
 
-    app.add_stylesheet("breathe.css")
+    breathe_css = "breathe.css"
+    if (os.path.exists(os.path.join(app.confdir, "_static", breathe_css))):
+        app.add_stylesheet(breathe_css)
 
     doxygen_handle = AutoDoxygenProcessHandle(
         path_handler,
@@ -1024,4 +1048,3 @@ def setup(app):
     app.connect("env-get-outdated", file_state_cache.get_outdated)
 
     app.connect("env-purge-doc", file_state_cache.purge_doc)
-
