@@ -8,8 +8,8 @@ from xml.dom import minidom
 from xml.dom import Node
 from xml.parsers.expat import ExpatError
 
-import compoundsuper as supermod
-from compoundsuper import MixedContainer
+from . import compoundsuper as supermod
+from .compoundsuper import MixedContainer
 
 
 class DoxygenTypeSub(supermod.DoxygenType):
@@ -890,6 +890,10 @@ class docParaTypeSub(supermod.docParaType):
             obj_ = supermod.docHeadingType.factory()
             obj_.build(child_)
             self.content.append(obj_)
+        elif child_.nodeType == Node.ELEMENT_NODE and nodeName_ == 'ulink':
+            obj_ = supermod.docURLLink.factory()
+            obj_.build(child_)
+            self.content.append(obj_)
 
 supermod.docParaType.subclass = docParaTypeSub
 # end class docParaTypeSub
